searchData={"items":[{"type":"module","title":"Moar.Assertions","doc":"ExUnit assertions.\n\nSee also: [siiibo/assert_match](https://github.com/siiibo/assert_match) which is similar to this module's\n`assert_eq` function but with pattern matching.","ref":"Moar.Assertions.html"},{"type":"function","title":"Moar.Assertions.assert_contains/2","doc":"Asserts that the `left` list or map contains all of the items in the `right` list or map,\nor contains the single `right` element if it's not a list or map. Returns `left` or raises `ExUnit.AssertionError`.\n\n```elixir\niex> assert_contains([1, 2, 3], [1, 3])\n[1, 2, 3]\n\niex> assert_contains(%{a: 1, b: 2, c: 3}, %{a: 1, c: 3})\n%{a: 1, b: 2, c: 3}\n\niex> assert_contains([1, 2, 3], 2)\n[1, 2, 3]\n```","ref":"Moar.Assertions.html#assert_contains/2"},{"type":"function","title":"Moar.Assertions.assert_eq/3","doc":"Asserts that the `left` and `right` values are equal. Returns the `left` value unless the assertion fails,\nor unless the `:returning` option is used.\n\nUses `assert left == right` under the hood, unless `left` is a string and\n`right` is a Regex, in which case they will be compared using the `=~` operator.\n\n_Style note: the authors prefer to use `assert` in most cases, using `assert_eq` only when the extra options\nare helpful or when they want to make assertions in a pipeline._\n\nOptions:\n\n* `except: ~w[a b]a` - ignore the given keys when comparing maps.\n* `ignore_order: boolean` - if the `left` and `right` values are lists, ignores the order when checking equality.\n* ~~`ignore_whitespace: :leading_and_trailing` - if the `left` and `right` values are strings, ignores leading and\n  trailing space when checking equality.~~ _deprecated: see `:whitespace` option_\n* `only: ~w[a b]a` - only consider the given keys when comparing maps.\n* `returning: value` - returns `value` if the assertion passes, rather than returning the `left` value.\n* `whitespace: :squish` - when `left` and `right` are strings, squishes via `Moar.String.squish/1` before comparing.\n* `whitespace: :trim` - when `left` and `right` are strings, trims via `String.trim/1` before comparing.\n* `within: delta` - asserts that the `left` and `right` values are within `delta` of each other.\n* `within: {delta, time_unit}` - like `within: delta` but performs time comparisons in the specified `time_unit`.\n  See `Moar.Duration` for more about time units. If `left` and `right` are strings, they are parsed as ISO8601 dates.","ref":"Moar.Assertions.html#assert_eq/3"},{"type":"function","title":"Examples - Moar.Assertions.assert_eq/3","doc":"```elixir\niex> import Moar.Assertions\n\niex> %{a: 1} |> Map.put(:b, 2) |> assert_eq(%{a: 1, b: 2})\n%{a: 1, b: 2}\n\niex> assert_eq(%{a: 1, b: 2, c: 3}, %{a: 1, b: 100, c: 3}, except: [:b])\n%{a: 1, b: 2, c: 3}\n\niex> assert_eq([1, 2], [2, 1], ignore_order: true)\n[1, 2]\n\niex> assert_eq(%{a: 1, b: 2, c: 3}, %{a: 1, b: 100, c: 3}, only: [:a, :c])\n%{a: 1, b: 2, c: 3}\n\niex> map = %{a: 1, b: 2}\niex> map |> Map.get(:a) |> assert_eq(1, returning: map)\n%{a: 1, b: 2}\n\niex> assert_eq(\"foo bar\", \"  foo bar\\n\", whitespace: :trim)\n\"foo bar\"\n\niex> assert_eq(4/28, 0.14, within: 0.01)\n0.14285714285714285\n\niex> inserted_at = ~U[2022-01-02 03:00:00Z]\niex> updated_at = ~U[2022-01-02 03:04:00Z]\niex> assert_eq(inserted_at, updated_at, within: {10, :minute})\n~U[2022-01-02 03:00:00Z]\n\niex> inserted_at = \"2022-01-02T03:00:00Z\"\niex> updated_at = \"2022-01-02T03:04:00Z\"\niex> assert_eq(inserted_at, updated_at, within: {10, :minute})\n\"2022-01-02T03:00:00Z\"\n```","ref":"Moar.Assertions.html#assert_eq/3-examples"},{"type":"function","title":"Moar.Assertions.assert_recent/2","doc":"Asserts that `datetime` is within `recency` of now (in UTC), returning `datetime` if the assertion succeeeds.\n\nUses `assert_eq(datetime, now, within: recency)` under the hood.\n\n* `datetime` can be a `DateTime`, a `NaiveDateTime`, or an ISO8601-formatted UTC datetime string.\n* `recency` is a `Moar.Duration` and defaults to `{10, :second}`.\n\n```elixir\niex> five_seconds_ago = Moar.DateTime.add(DateTime.utc_now(), {-5, :second})\niex> assert_recent five_seconds_ago\n\niex> twenty_seconds_ago = Moar.DateTime.add(DateTime.utc_now(), {-20, :second})\niex> assert_recent twenty_seconds_ago, {25, :second}\n\n```","ref":"Moar.Assertions.html#assert_recent/2"},{"type":"macro","title":"Moar.Assertions.assert_that/2","doc":"Asserts that a pre-condition and a post-condition are true after performing an action,\nreturning the result of the action.\n\nTo use an anonymous function as the action, wrap it in parentheses and call it with `.()`.","ref":"Moar.Assertions.html#assert_that/2"},{"type":"macro","title":"Examples - Moar.Assertions.assert_that/2","doc":"```elixir\niex> {:ok, agent} = Agent.start(fn -> 0 end)\n...>\niex> assert_that Agent.update(agent, fn s -> s + 1 end),\n...>     changes: Agent.get(agent, fn s -> s end),\n...>     from: 0,\n...>     to: 1\n:ok\n...>\niex> assert_that Agent.update(agent, fn s -> s + 1 end),\n...>     changes: Agent.get(agent, fn s -> s end),\n...>     to: 2\n:ok\n...>\niex> assert_that (fn -> Agent.update(agent, fn s -> s + 1 end) end).(),\n...>     changes: Agent.get(agent, fn s -> s end),\n...>     to: 3\n:ok\n```","ref":"Moar.Assertions.html#assert_that/2-examples"},{"type":"macro","title":"Moar.Assertions.refute_that/2","doc":"Refute that a condition is changed after performing an action, returning the result of the action.","ref":"Moar.Assertions.html#refute_that/2"},{"type":"macro","title":"Examples - Moar.Assertions.refute_that/2","doc":"```elixir\niex> {:ok, agent} = Agent.start(fn -> 0 end)\n...>\niex> refute_that Function.identity(1),\n...>     changes: Agent.get(agent, fn s -> s end)\n1\n\niex> refute_that Function.identity(5),\n...>     changes: %{a: 1}\n5\n```","ref":"Moar.Assertions.html#refute_that/2-examples"},{"type":"type","title":"Moar.Assertions.assert_eq_opts/0","doc":"","ref":"Moar.Assertions.html#t:assert_eq_opts/0"},{"type":"module","title":"Moar.Atom","doc":"Atom-related functions.","ref":"Moar.Atom.html"},{"type":"function","title":"Moar.Atom.atomize/1","doc":"Given an atom, returns the atom. Given a string, converts it into an atom, converting non-alphanumeric characters\ninto underscores (via `Moar.String.slug/2`).","ref":"Moar.Atom.html#atomize/1"},{"type":"function","title":"Examples - Moar.Atom.atomize/1","doc":"```\niex> Moar.Atom.atomize(:my_atom)\n:my_atom\n\niex> Moar.Atom.atomize(\"my-atom\")\n:my_atom\n```","ref":"Moar.Atom.html#atomize/1-examples"},{"type":"function","title":"Moar.Atom.ensure_existing_atoms/1","doc":"Given a list of strings, returns `:ok` if all values have a corresponding atom that already exists.\nOtherwise, it returns an error tuple with a list of strings that don't have corresponding atoms.\n\nAny atom included in the argument will be considered an existing atom.","ref":"Moar.Atom.html#ensure_existing_atoms/1"},{"type":"function","title":"Examples - Moar.Atom.ensure_existing_atoms/1","doc":"```\niex> :existing_atom\niex> Moar.Atom.ensure_existing_atoms([\"existing_atom\", :another_existing_atom])\n:ok\n\niex> :existing_atom\niex> Moar.Atom.ensure_existing_atoms([\"existing_atom\", :another_existing_atom, \"some_nonexisting_atom\"])\n{:error, [\"some_nonexisting_atom\"]}\n```","ref":"Moar.Atom.html#ensure_existing_atoms/1-examples"},{"type":"function","title":"Moar.Atom.existing_atom?/1","doc":"Given a string, returns `true` if a corresponding atom has been previously defined. Otherwise, returns `false`.\n\nGiven an atom, returns `true`.","ref":"Moar.Atom.html#existing_atom?/1"},{"type":"function","title":"Examples - Moar.Atom.existing_atom?/1","doc":"```\niex> :existing_atom\niex> Moar.Atom.existing_atom?(\"existing_atom\")\ntrue\n\niex> Moar.Atom.existing_atom?(:another_existing_atom)\ntrue\n\niex> Moar.Atom.existing_atom?(\"some_nonexisting_atom\")\nfalse\n```","ref":"Moar.Atom.html#existing_atom?/1-examples"},{"type":"function","title":"Moar.Atom.from_string/1","doc":"Converts a string to an atom (via `String.to_atom/1`), and returns atoms unchanged.\n\nUseful when you aren't sure ahead of time whether you have a string or an atom.","ref":"Moar.Atom.html#from_string/1"},{"type":"function","title":"Examples - Moar.Atom.from_string/1","doc":"```elixir\niex> Moar.Atom.from_string(\"foo\")\n:foo\n\niex> Moar.Atom.from_string(:bar)\n:bar\n\niex> Moar.Atom.from_string(nil)\n** (ArgumentError) Unable to convert nil into an atom\n```","ref":"Moar.Atom.html#from_string/1-examples"},{"type":"function","title":"Moar.Atom.to_existing_atom/1","doc":"Converts a string to an existing atom (via `String.to_existing_atom/1`), and returns atoms unchanged.\n\nUseful when you aren't sure ahead of time whether you have a string or an atom.","ref":"Moar.Atom.html#to_existing_atom/1"},{"type":"function","title":"Examples - Moar.Atom.to_existing_atom/1","doc":"```elixir\niex> Moar.Atom.to_existing_atom(\"foo\")\n:foo\n\niex> assert_raise ArgumentError, fn ->\n...>   Moar.Atom.to_existing_atom(\"sadfasfsfasf\")\n...> end\n\niex> Moar.Atom.to_existing_atom(:baz)\n:baz\n\niex> Moar.Atom.to_existing_atom(nil)\nnil\n```","ref":"Moar.Atom.html#to_existing_atom/1-examples"},{"type":"function","title":"Moar.Atom.to_string/1","doc":"Converts an atom to a string (via `Atom.to_string/1`), and returns strings unchanged.\n\nUseful when you aren't sure ahead of time whether you have a string or an atom.","ref":"Moar.Atom.html#to_string/1"},{"type":"function","title":"Examples - Moar.Atom.to_string/1","doc":"```elixir\niex> Moar.Atom.to_string(\"foo\")\n\"foo\"\n\niex> Moar.Atom.to_string(:bar)\n\"bar\"\n\niex> Moar.Atom.to_string(nil)\n** (ArgumentError) Unable to convert nil into a string\n```","ref":"Moar.Atom.html#to_string/1-examples"},{"type":"module","title":"Moar.DateTime","doc":"DateTime-related functions. See also `Moar.NaiveDateTime`.","ref":"Moar.DateTime.html"},{"type":"function","title":"Moar.DateTime.add/2","doc":"Adds `duration` to `date_time`.\n\nSee also `subtract/1` and `Moar.NaiveDateTime.add/2`.\n\n> ##","ref":"Moar.DateTime.html#add/2"},{"type":"function","title":"Note {: .info} - Moar.DateTime.add/2","doc":">\n> This function is naive and intentionally doesn't account for real-world calendars and all of their complexity,\n> such as leap years, leap days, daylight saving time, past and future calendar oddities, etc.\n>\n> As [\"Falsehoods programmers believe about time\"](https://gist.github.com/timvisee/fcda9bbdff88d45cc9061606b4b923ca)\n> says, \"If you think you understand everything about time, you're probably doing it wrong.\"\n>\n> See [`Cldr.Calendar.plus/2`](https://hexdocs.pm/ex_cldr_calendars/Cldr.Calendar.html#plus/2) for one example\n> of a function that is far more likely to be correct.\n\n```elixir\niex> start = ~U[2022-01-01T00:00:00.000Z]\niex> Moar.DateTime.add(start, {3, :minute})\n~U[2022-01-01T00:03:00.000Z]\n```","ref":"Moar.DateTime.html#add/2-note-info"},{"type":"function","title":"Moar.DateTime.between?/2","doc":"Returns true if `date_time` is inclusively inside `range` which is a tuple containing a start datetime and an end\ndatetime.\n\n```elixir\niex> Moar.DateTime.between?(~U[2022-01-01T02:00:00Z], {~U[2022-01-01T01:00:00Z], ~U[2022-01-01T03:00:00Z]})\ntrue\n\niex> Moar.DateTime.between?(~U[2022-01-01T04:00:00Z], {~U[2022-01-01T01:00:00Z], ~U[2022-01-01T03:00:00Z]})\nfalse\n```","ref":"Moar.DateTime.html#between?/2"},{"type":"function","title":"Moar.DateTime.from_iso8601!/1","doc":"Like `DateTime.from_iso8601/1` but raises if the string cannot be parsed.\n\n```elixir\niex> Moar.DateTime.from_iso8601!(\"2022-01-01T00:00:00Z\")\n~U[2022-01-01T00:00:00Z]\n\niex> Moar.DateTime.from_iso8601!(\"2022-01-01T00:00:00+0800\")\n** (ArgumentError) Expected \"2022-01-01T00:00:00+0800\" to have a UTC offset of 0, but was: 28800\n\niex> Moar.DateTime.from_iso8601!(\"Next Thursday after lunch\")\n** (ArgumentError) Invalid ISO8601 format: \"Next Thursday after lunch\"\n```","ref":"Moar.DateTime.html#from_iso8601!/1"},{"type":"function","title":"Moar.DateTime.recent?/2","doc":"Returns true if `date_time` is now or in the past and no older than `duration` ago (which defaults to 1 minute).\nReturns false if `date_time` is in the future.\n\n```elixir\niex> Moar.DateTime.recent?(Moar.DateTime.utc_now(minus: {30, :second}))\ntrue\n\niex> Moar.DateTime.recent?(Moar.DateTime.utc_now(minus: {5, :minute}))\nfalse\n\niex> Moar.DateTime.recent?(Moar.DateTime.utc_now(minus: {5, :minute}), {1, :hour})\ntrue\n\niex> Moar.DateTime.recent?(Moar.DateTime.utc_now(plus: {5, :second}))\nfalse\n```","ref":"Moar.DateTime.html#recent?/2"},{"type":"function","title":"Moar.DateTime.subtract/2","doc":"Subtracts `duration` from `date_time`.\n\nSee also `add/1` and `Moar.NaiveDateTime.subtract/2`.\n\n> ##","ref":"Moar.DateTime.html#subtract/2"},{"type":"function","title":"Note {: .info} - Moar.DateTime.subtract/2","doc":">\n> This function is naive and intentionally doesn't account for real-world calendars and all of their complexity,\n> such as leap years, leap days, daylight saving time, past and future calendar oddities, etc.\n>\n> As [\"Falsehoods programmers believe about time\"](https://gist.github.com/timvisee/fcda9bbdff88d45cc9061606b4b923ca)\n> says, \"If you think you understand everything about time, you're probably doing it wrong.\"\n>\n> See [`Cldr.Calendar.minus/4`](https://hexdocs.pm/ex_cldr_calendars/Cldr.Calendar.html#minus/4) for one example\n> of a function that is far more likely to be correct.\n\n```elixir\niex> start = ~U[2022-01-01T00:03:00.000Z]\niex> Moar.DateTime.subtract(start, {3, :minute})\n~U[2022-01-01T00:00:00.000Z]\n```","ref":"Moar.DateTime.html#subtract/2-note-info"},{"type":"function","title":"Moar.DateTime.to_iso8601_rounded/1","doc":"Like `DateTime.to_iso8601/1` but rounds to the nearest second first.\n\n```elixir\niex> Moar.DateTime.to_iso8601_rounded(~U[2022-01-01T01:02:03.456789Z])\n\"2022-01-01T01:02:03Z\"\n```","ref":"Moar.DateTime.html#to_iso8601_rounded/1"},{"type":"function","title":"Moar.DateTime.utc_now/1","doc":"Returns the current UTC time plus or minus the given duration.\n\n```elixir\niex> Moar.DateTime.utc_now(plus: {10500, :millisecond})\n...> |> Moar.Duration.format([:approx, :from_now])\n\"10 seconds from now\"\n\niex> Moar.DateTime.utc_now(minus: {10, :second})\n...> |> Moar.Duration.format([:approx, :ago])\n\"10 seconds ago\"\n```","ref":"Moar.DateTime.html#utc_now/1"},{"type":"function","title":"Moar.DateTime.within?/2","doc":"Returns true if `date_time` is no older than `duration` ago, and no later than `duration` from now.\n\n```elixir\niex> Moar.DateTime.within?(Moar.DateTime.utc_now(minus: {30, :second}), {1, :minute})\ntrue\n\niex> Moar.DateTime.within?(Moar.DateTime.utc_now(plus: {30, :second}), {1, :minute})\ntrue\n\niex> Moar.DateTime.within?(Moar.DateTime.utc_now(minus: {5, :minute}), {1, :minute})\nfalse\n\niex> Moar.DateTime.within?(Moar.DateTime.utc_now(plus: {5, :minute}), {1, :minute})\nfalse\n```","ref":"Moar.DateTime.html#within?/2"},{"type":"protocol","title":"Moar.Difference","doc":"A protocol that defines `diff/2` for finding the difference between two terms.\n\nIncludes implementations for `DateTime`, `NaiveDateTime`, and `BitString`.","ref":"Moar.Difference.html"},{"type":"function","title":"Moar.Difference.diff/2","doc":"Returns the difference between `a` and `b`.\n\nThe fallback implementation uses `Kernel.-/2` to subtract `b` from `a`.","ref":"Moar.Difference.html#diff/2"},{"type":"type","title":"Moar.Difference.t/0","doc":"All the types that implement this protocol.","ref":"Moar.Difference.html#t:t/0"},{"type":"module","title":"Moar.Duration","doc":"A duration is a `{time, unit}` tuple.\n\nThe time is a number and the unit is one of:\n* `:nanosecond`\n* `:microsecond`\n* `:millisecond`\n* `:second`\n* `:minute`\n* `:hour`\n* `:day`\n* `:approx_month` (30 days)\n* `:approx_year` (360 days)\n\n> ##","ref":"Moar.Duration.html"},{"type":"module","title":"Note {: .info} - Moar.Duration","doc":">\n> This module is naive and intentionally doesn't account for real-world calendars and all of their complexity,\n> such as leap years, leap days, daylight saving time, past and future calendar oddities, etc.\n>\n> As [\"Falsehoods programmers believe about time\"](https://gist.github.com/timvisee/fcda9bbdff88d45cc9061606b4b923ca)\n> says, \"If you think you understand everything about time, you're probably doing it wrong.\"\n>\n> See [`Cldr.Calendar.Duration`](https://hexdocs.pm/ex_cldr_calendars/Cldr.Calendar.Duration.html) for one example\n> of a full-featured library that is far more likely to be correct.","ref":"Moar.Duration.html#module-note-info"},{"type":"function","title":"Moar.Duration.ago/1","doc":"Returns the duration between `datetime` and now, in the largest possible unit.\n\n`datetime` can be an ISO8601-formatted string, a `DateTime`, or a `NaiveDateTime`.\n\nSee also `from_now/1`.\n\n```elixir\niex> DateTime.utc_now()\n...> |> Moar.DateTime.subtract({121, :minute})\n...> |> Moar.Duration.ago()\n...> |> Moar.Duration.shift(:minute)\n{121, :minute}\n```","ref":"Moar.Duration.html#ago/1"},{"type":"function","title":"Moar.Duration.approx/1","doc":"Shifts `duration` to an approximately equal duration that's simpler. For example, `{121, :second}` would get\nshifted to `{2, :minute}`.\n\n> ##","ref":"Moar.Duration.html#approx/1"},{"type":"function","title":"Warning {: .warning} - Moar.Duration.approx/1","doc":">\n> This function is lossy because it intentionally loses precision.\n\nIf the time value of the duration is exactly 1, the duration is returned unchanged: `{1, :minute}` => `{1, :minute}`.\nOtherwise, the duration is shifted to the highest unit where the time value is >= 2.\n\n```elixir\niex> Moar.Duration.approx({1, :minute})\n{1, :minute}\n\niex> Moar.Duration.approx({7300, :second})\n{2, :hour}\n```","ref":"Moar.Duration.html#approx/1-warning-warning"},{"type":"function","title":"Moar.Duration.between/2","doc":"Returns the duration between `earlier` and `later`, in the largest possible unit.\n\n`earlier` and `later` can be ISO8601-formatted strings, `DateTime`s, or `NaiveDateTime`s.\n\n```elixir\niex> earlier = ~U[2020-01-01T00:00:00.000000Z]\niex> later = ~U[2020-01-01T02:01:00.000000Z]\niex> Moar.Duration.between(earlier, later)\n{121, :minute}\n```","ref":"Moar.Duration.html#between/2"},{"type":"function","title":"Moar.Duration.convert/2","doc":"Converts a `{duration, time_unit}` tuple into a numeric duration, rounding down to the nearest whole number.\n\n> ##","ref":"Moar.Duration.html#convert/2"},{"type":"function","title":"Warning {: .warning} - Moar.Duration.convert/2","doc":">\n> This function is lossy because it rounds down to the nearest whole number.\n\nUses `System.convert_time_unit/3` under the hood; see its documentation for more details.\n\nIt is similar to `shift/1` but this function returns an integer value, while `shift/1` returns a duration tuple.\n\n```elixir\niex> Moar.Duration.convert({121, :second}, :minute)\n2\n```","ref":"Moar.Duration.html#convert/2-warning-warning"},{"type":"function","title":"Moar.Duration.format/4","doc":"Formats a duration in either a long or short style, with optional transformers and an optional suffix.\n\n* The first argument is a duration tuple, unless one of the transformers is `:ago`, in which case\n  it can be a `DateTime`, `NaiveDateTime`, or an ISO8601-formatted string.\n* The second argument is optional and is the style, transformer, list of transformers, or suffix.\n* The third argument is optional and is the transformer, list of transformers, or suffix.\n* The fourth argument is optional and is the suffix.\n\nStyles:\n  * `:long`, which formats like `\"25 seconds\"`.\n  * `:short`, which formats like `\"25s\"`.\n  * Defaults to `:long`\n  \nTransformers:\n  * `:ago` transforms via `ago/1`\n  * `:approx` transforms via `approx/1`\n  * `:humanize` transforms via `humanize/1`\n  * If no transformers are specified, no transformations are applied.\n  \nSuffix:\n  * A string that will be appended to the formatted result.\n  * If the `:ago` transformer is specified and a suffix is not specified, the suffix will default to `\"ago\"`.\n    To use the \"ago\" transformer with no suffix, specify an empty string as the suffix (`nil` will not suffice).\n  \n```elixir\niex> Moar.Duration.format({1, :second})\n\"1 second\"\n\niex> Moar.Duration.format({120, :second})\n\"120 seconds\"\n\niex> Moar.Duration.format({120, :second}, :long)\n\"120 seconds\"\n\niex> Moar.Duration.format({120, :second}, :short)\n\"120s\"\n\niex> Moar.Duration.format({120, :second}, \"yonder\")\n\"120 seconds yonder\"\n\niex> Moar.Duration.format({120, :second}, :humanize)\n\"2 minutes\"\n\niex> Moar.Duration.format({120, :second}, :humanize, \"yonder\")\n\"2 minutes yonder\"\n\niex> Moar.Duration.format({310, :second})\n\"310 seconds\"\n\niex> Moar.Duration.format({310, :second}, :approx)\n\"5 minutes\"\n\niex> DateTime.utc_now()\n...> |> Moar.DateTime.add({-310, :second})\n...> |> Moar.Duration.format(:short, [:ago, :approx], \"henceforth\")\n\"5m henceforth\"\n```","ref":"Moar.Duration.html#format/4"},{"type":"function","title":"Moar.Duration.from_now/1","doc":"Returns the duration between now and `datetime`, in the largest possible unit.\n\n`datetime` can be an ISO8601-formatted string, a `DateTime`, or a `NaiveDateTime`.\n\nSee also `ago/1`.\n\n```elixir\niex> DateTime.utc_now()\n...> |> Moar.DateTime.add({121, :minute})\n...> |> Moar.Duration.from_now()\n...> |> Moar.Duration.approx()\n{2, :hour}\n```","ref":"Moar.Duration.html#from_now/1"},{"type":"function","title":"Moar.Duration.humanize/1","doc":"If possible, shifts `duration` to a higher time unit that is more readable to a human. Returns `duration`\nunchanged if it cannot be exactly shifted.\n\n```elixir\niex> Moar.Duration.humanize({60000, :millisecond})\n{1, :minute}\n\niex> Moar.Duration.humanize({48, :hour})\n{2, :day}\n\niex> Moar.Duration.humanize({49, :hour})\n{49, :hour}\n```","ref":"Moar.Duration.html#humanize/1"},{"type":"function","title":"Moar.Duration.shift/2","doc":"Shifts `duration` to `time_unit`. It is similar to `convert/1` but this function returns a duration tuple,\nwhile `convert/1` just returns an integer value.\n\n> ##","ref":"Moar.Duration.html#shift/2"},{"type":"function","title":"Warning {: .warning} - Moar.Duration.shift/2","doc":">\n> This function is lossy because it rounds down to the nearest whole number.\n\n```elixir\niex> Moar.Duration.shift({121, :second}, :minute)\n{2, :minute}\n```","ref":"Moar.Duration.html#shift/2-warning-warning"},{"type":"function","title":"Moar.Duration.shift_down/1","doc":"Shifts `duration` to the next smaller unit. Raises if it's already at the smallest unit (nanosecond).\n\n```elixir\niex> Moar.Duration.shift_down({1, :hour})\n{60, :minute}\n```","ref":"Moar.Duration.html#shift_down/1"},{"type":"function","title":"Moar.Duration.shift_up/1","doc":"Shifts `duration` to the next larger unit. Raises if it's already at the largest unit (approx_year).\nRounds the result towards zero.\n\n> ##","ref":"Moar.Duration.html#shift_up/1"},{"type":"function","title":"Warning {: .warning} - Moar.Duration.shift_up/1","doc":">\n> This function is lossy because it rounds down to the nearest whole number.\n\n```elixir\niex> Moar.Duration.shift_up({60, :minute})\n{1, :hour}\n\niex> Moar.Duration.shift_up({125, :minute})\n{2, :hour}\n```","ref":"Moar.Duration.html#shift_up/1-warning-warning"},{"type":"function","title":"Moar.Duration.to_string/1","doc":"Shortcut to `format(duration, :long)`. See `format/4`.","ref":"Moar.Duration.html#to_string/1"},{"type":"type","title":"Moar.Duration.date_time_ish/0","doc":"","ref":"Moar.Duration.html#t:date_time_ish/0"},{"type":"type","title":"Moar.Duration.format_style/0","doc":"","ref":"Moar.Duration.html#t:format_style/0"},{"type":"type","title":"Moar.Duration.format_transformer/0","doc":"","ref":"Moar.Duration.html#t:format_transformer/0"},{"type":"type","title":"Moar.Duration.format_transformers/0","doc":"","ref":"Moar.Duration.html#t:format_transformers/0"},{"type":"type","title":"Moar.Duration.t/0","doc":"","ref":"Moar.Duration.html#t:t/0"},{"type":"type","title":"Moar.Duration.time_unit/0","doc":"","ref":"Moar.Duration.html#t:time_unit/0"},{"type":"module","title":"Moar.Enum","doc":"Enum-related functions.","ref":"Moar.Enum.html"},{"type":"function","title":"Moar.Enum.at!/3","doc":"Like `Enum.at/3` but raises if `index` is out of bounds.","ref":"Moar.Enum.html#at!/3"},{"type":"function","title":"Moar.Enum.compact/1","doc":"Removes nil elements from `enum`.","ref":"Moar.Enum.html#compact/1"},{"type":"function","title":"Moar.Enum.find_indices/3","doc":"Returns the indices of `elements` in `enum`, using `fun` for comparisons (defaulting to `Kernel.==/2`)\n\n```elixir\niex> Moar.Enum.find_indices(~w[apple banana cherry], ~w[cherry apple])\n[2, 0]\n\niex> Moar.Enum.find_indices(~w[apple banana cherry], ~w[CHERRY APPLE], fn a, b ->\n...>   String.downcase(a) == String.downcase(b)\n...> end)\n[2, 0]\n```","ref":"Moar.Enum.html#find_indices/3"},{"type":"function","title":"Moar.Enum.find_indices!/3","doc":"Returns the indices of `elements` in `enum`, raising if any member of `elements` is not found.\n\n```elixir\niex> Moar.Enum.find_indices!(~w[apple banana cherry], ~w[cherry apple])\n[2, 0]\n\niex> Moar.Enum.find_indices!(~w[apple banana], ~w[cherry apple])\n** (RuntimeError) Element \"cherry\" not present in:\n[\"apple\", \"banana\"]\n```","ref":"Moar.Enum.html#find_indices!/3"},{"type":"function","title":"Moar.Enum.first!/1","doc":"Returns the first item of `enum`, or raises if it is empty.","ref":"Moar.Enum.html#first!/1"},{"type":"function","title":"Moar.Enum.index_by/2","doc":"Converts an enum into a map of maps indexed by the return value of `index_fun`.\nSee also the similar map-specific `Moar.Map.index_by/2`.\n\n```elixir\niex> Moar.Enum.index_by([%{name: \"Alice\", tid: \"alice\"}, %{name: \"Billy\", tid: \"billy\"}], & &1.tid)\n%{\"alice\" => %{name: \"Alice\", tid: \"alice\"}, \"billy\" => %{name: \"Billy\", tid: \"billy\"}}\n```","ref":"Moar.Enum.html#index_by/2"},{"type":"function","title":"Moar.Enum.into!/2","doc":"Like `Enum.into` but accepts `nil` as the first argument","ref":"Moar.Enum.html#into!/2"},{"type":"function","title":"Moar.Enum.is_map_or_keyword/1","doc":"Deprecated in favor of `map_or_keyword?/1`.","ref":"Moar.Enum.html#is_map_or_keyword/1"},{"type":"function","title":"Moar.Enum.is_map_or_nonempty_keyword/1","doc":"Deprecated in favor of `map_or_nonempty_keyword?/1`.","ref":"Moar.Enum.html#is_map_or_nonempty_keyword/1"},{"type":"function","title":"Moar.Enum.isort/1","doc":"Sorts `enum` case-insensitively. Uses `Enum.sort_by/3` under the hood.","ref":"Moar.Enum.html#isort/1"},{"type":"function","title":"Moar.Enum.isort_by/2","doc":"Sorts `enum` case-insensitively by `mapper` function. Uses `Enum.sort_by/3` under the hood.","ref":"Moar.Enum.html#isort_by/2"},{"type":"function","title":"Moar.Enum.lists_to_maps/2","doc":"Converts a list of lists to a list of maps with the given keys. The keys can be a list, or can be `:first_list`\nwhich uses the first list in `list_of_lists` as the keys and the remaining lists in `list_of_lists` as the values.\n\n```elixir\niex> Moar.Enum.lists_to_maps([[1, 2], [3, 4]], [\"a\", \"b\"])\n[%{\"a\" => 1, \"b\" => 2}, %{\"a\" => 3, \"b\" => 4}]\n\niex> csv = [[\"a\", \"b\"], [1, 2], [3, 4]]\niex> [headers | rows] = csv\niex> Moar.Enum.lists_to_maps(rows, headers)\n[%{\"a\" => 1, \"b\" => 2}, %{\"a\" => 3, \"b\" => 4}]\n\niex> csv = [[\"a\", \"b\"], [1, 2], [3, 4]]\niex> Moar.Enum.lists_to_maps(csv, :first_list)\n[%{\"a\" => 1, \"b\" => 2}, %{\"a\" => 3, \"b\" => 4}]\n```","ref":"Moar.Enum.html#lists_to_maps/2"},{"type":"function","title":"Moar.Enum.map_or_keyword?/1","doc":"Returns true if the value is a map or a keyword list. This uses standard Elixir functions for determining\nif a term is a map or a keyword, and therefore counts an empty list as a keyword list. See also\n`map_or_nonempty_keyword?/1`.\n\nThis cannot be used as a guard because it uses `Keyword.keyword?` under the hood. Also, because of that,\nit might scan an entire list to see if it's a keyword list, so it might be expensive.","ref":"Moar.Enum.html#map_or_keyword?/1"},{"type":"function","title":"Moar.Enum.map_or_nonempty_keyword?/1","doc":"Like `map_or_keyword?/1` but returns false if the term is an empty list.","ref":"Moar.Enum.html#map_or_nonempty_keyword?/1"},{"type":"function","title":"Moar.Enum.take_at/2","doc":"Returns a list of elements at the given indices, in the given order. If `:all` is given instead of a list of indices,\nthe entire enum is returned.\n\n```elixir\niex> Moar.Enum.take_at([\"A\", \"B\", \"C\"], [0, 2])\n[\"A\", \"C\"]\n\niex> Moar.Enum.take_at([\"A\", \"B\", \"C\"], [2, 0])\n[\"C\", \"A\"]\n\niex> Moar.Enum.take_at([\"A\", \"B\", \"C\"], :all)\n[\"A\", \"B\", \"C\"]\n```","ref":"Moar.Enum.html#take_at/2"},{"type":"function","title":"Moar.Enum.test_ids/2","doc":"Returns `:test_id` fields from `enumerable`.\n\nThis unusual function exists because the authors of Moar use test_ids extensively in tests.\nSee also `tids/2`.","ref":"Moar.Enum.html#test_ids/2"},{"type":"function","title":"Moar.Enum.tids/2","doc":"Returns `:tid` fields from `enumerable`.\n\nThis unusual function exists because the authors of Moar use tids (test IDs) extensively in tests.\nSee also `test_ids/2`.","ref":"Moar.Enum.html#tids/2"},{"type":"type","title":"Moar.Enum.test_ids_opts/0","doc":"","ref":"Moar.Enum.html#t:test_ids_opts/0"},{"type":"type","title":"Moar.Enum.tids_opts/0","doc":"","ref":"Moar.Enum.html#t:tids_opts/0"},{"type":"module","title":"Moar.File","doc":"File-related functions.","ref":"Moar.File.html"},{"type":"function","title":"Moar.File.new_tempfile_path/1","doc":"Returns a randomly-named path in the system temp directory. Does not actually create the file at the path.\n\nFor example, `Moar.File.new_tempfile_path(\".txt\")` would return something like\n`\"/var/folders/3r/n09zyyy16yddj9m2d1ppzvt40000gn/T/XY7V5E26DV.txt\"`.\n\n```elixir\niex> Moar.File.new_tempfile_path(\".txt\") |> File.exists?()\nfalse\n```","ref":"Moar.File.html#new_tempfile_path/1"},{"type":"function","title":"Moar.File.write_tempfile/2","doc":"Writes `contents` to a new temp file with extension `file_extension`, and returns the path to the file.\n\nFor example, `Moar.File.write_tempfile(\"hello\", \".txt\")` would create a file in a path something like\n`/var/folders/3r/n09zyyy16yddj9m2d1ppzvt40000gn/T/RHUEJCCH22.txt` with the contents `hello`.\n\n```elixir\niex> Moar.File.write_tempfile(\"hello\", \".txt\") |> File.read!()\n\"hello\"\n```","ref":"Moar.File.html#write_tempfile/2"},{"type":"module","title":"Moar.Integer","doc":"Integer-related functions.","ref":"Moar.Integer.html"},{"type":"function","title":"Moar.Integer.compare/2","doc":"Compares two integers, returning `:lt`, `:eq`, or `:gt`.\n\n```\niex> Moar.Integer.compare(1, 3)\n:lt\n\niex> Moar.Integer.compare(1, -3)\n:gt\n\niex> Moar.Integer.compare(-1, -1)\n:eq\n\niex> Enum.sort([5, 6, 0, -1], Moar.Integer)\n[-1, 0, 5, 6]\n```","ref":"Moar.Integer.html#compare/2"},{"type":"module","title":"Moar.List","doc":"","ref":"Moar.List.html"},{"type":"function","title":"Moar.List.to_keyword/2","doc":"Converts a list to a keyword list, setting the value to `default_value`. The list can be a regular list, a keyword\nlist (which is a no-op), or a list that's a mix of terms and keywords (e.g., `[:a, :b, c: 3, d: 4]`). The default\nvalue can be a term or a function that accepts a key and is expected to return a value.\n\n```\niex> Moar.List.to_keyword([:a, :b, c: 3, d: 4])\n[a: nil, b: nil, c: 3, d: 4]\n\niex> Moar.List.to_keyword([:a, :b, c: 3, d: 4], 1)\n[a: 1, b: 1, c: 3, d: 4]\n\niex> Moar.List.to_keyword([:ant, :bird, cheetah: 7, dingo: 5], fn key -> key |> to_string() |> String.length() end)\n[ant: 3, bird: 4, cheetah: 7, dingo: 5]\n```","ref":"Moar.List.html#to_keyword/2"},{"type":"function","title":"Moar.List.to_sentence/2","doc":"Converts a list to a comma-separated list that has \"and\" before the last item. The items in the list are converted\nto strings via a mapper function, which defaults to `Kernel.to_string/1`.\n\n```\niex> Moar.List.to_sentence([])\n\"\"\n\niex> Moar.List.to_sentence([\"ant\"])\n\"ant\"\n\niex> Moar.List.to_sentence([\"ant\", :bat])\n\"ant and bat\"\n\niex> Moar.List.to_sentence([\"ant\", :bat, \"cat\"])\n\"ant, bat, and cat\"\n\niex> Moar.List.to_sentence([\"ant\", \"bat\", \"cat\"], &String.upcase/1)\n\"ANT, BAT, and CAT\"\n```","ref":"Moar.List.html#to_sentence/2"},{"type":"function","title":"Moar.List.unwrap!/1","doc":"Returns the argument if it is not a list. If it is a list, returns the only item\nin the list, or raises if the list is empty or has more than one item.\n\n```\niex> Moar.List.unwrap!([5])\n5\n\niex> Moar.List.unwrap!(5)\n5\n\niex> Moar.List.unwrap!([])\n** (FunctionClauseError) no function clause matching in Moar.List.unwrap!/1\n\niex> Moar.List.unwrap!([5, 9])\n** (FunctionClauseError) no function clause matching in Moar.List.unwrap!/1\n```","ref":"Moar.List.html#unwrap!/1"},{"type":"module","title":"Moar.Map","doc":"Map-related functions.","ref":"Moar.Map.html"},{"type":"function","title":"Moar.Map.atomize_key/3","doc":"Converts `key` in `map` to an atom, optionally transforming the value with `value_transformer`.\n\nRaises if `key` is a string and `map` already has an atomized version of that key.\n\n```elixir\niex> Moar.Map.atomize_key(%{\"number-one\" => \"one\", \"number-two\" => \"two\"}, \"number-one\")\n%{:number_one => \"one\", \"number-two\" => \"two\"}\n\niex> Moar.Map.atomize_key(%{\"number-one\" => \"one\", \"number-two\" => \"two\"}, \"number-one\", &String.upcase/1)\n%{:number_one => \"ONE\", \"number-two\" => \"two\"}\n```","ref":"Moar.Map.html#atomize_key/3"},{"type":"function","title":"Moar.Map.atomize_key!/3","doc":"Like `atomize_key/3` but raises if `key` is not in `map`.","ref":"Moar.Map.html#atomize_key!/3"},{"type":"function","title":"Moar.Map.atomize_keys/1","doc":"Converts keys in `map` to atoms.\n\nRaises if converting a key from a string to an atom would result in a key conflict.\n\n```elixir\niex> Moar.Map.atomize_keys(%{\"a\" => 1, \"b\" => 2})\n%{a: 1, b: 2}\n```","ref":"Moar.Map.html#atomize_keys/1"},{"type":"function","title":"Moar.Map.compact/1","doc":"Removes keys from a map where the value is nil.\n\n```elixir\niex> Moar.Map.compact(%{a: 1, b: nil, c: \"\"})\n%{a: 1, c: \"\"}\n```","ref":"Moar.Map.html#compact/1"},{"type":"function","title":"Moar.Map.deep_atomize_keys/1","doc":"Converts keys to atoms, traversing through descendant lists and maps.\n\nRaises if converting a key from a string to an atom would result in a key conflict.\n\n```elixir\niex> Moar.Map.deep_atomize_keys(%{\"a\" => %{\"aa\" => 1}, \"b\" => [%{\"bb\" => 2}, %{\"bbb\" => 3}]})\n%{a: %{aa: 1}, b: [%{bb: 2}, %{bbb: 3}]}\n```","ref":"Moar.Map.html#deep_atomize_keys/1"},{"type":"function","title":"Moar.Map.deep_merge/3","doc":"Deeply merges two maps into a single map. (It will also accept keyword lists and convert them to maps, as long\nas they are not empty lists.)\n\nOptionally accepts `conflict_fn` which gets called when both enumerables have values at the same keypath.\nIt receives the conflicting values from each map and is expected to return the winning value.\n\n```elixir\niex> Moar.Map.deep_merge(%{fruit: %{apples: 3, bananas: 5}, veggies: %{carrots: 10}}, [fruit: [cherries: 20]])\n%{fruit: %{apples: 3, bananas: 5, cherries: 20}, veggies: %{carrots: 10}}\n\niex> Moar.Map.deep_merge(%{a: %{b: 1}}, %{a: %{b: 2}})\n%{a: %{b: 2}}\n\niex> Moar.Map.deep_merge(%{a: %{b: 1}}, %{a: %{b: 2}}, fn x, _y -> x end)\n%{a: %{b: 1}}\n\niex> Moar.Map.deep_merge(%{a: %{b: 1}}, %{a: %{b: 2}}, fn x, y -> [x, y] end)\n%{a: %{b: [1, 2]}}\n```","ref":"Moar.Map.html#deep_merge/3"},{"type":"function","title":"Moar.Map.deep_take/2","doc":"Takes keys from nested maps.\n\n```elixir\niex> Moar.Map.deep_take(%{a: 1, b: %{c: 2, d: 3}, z: 9}, [:a, b: [:c]])\n%{a: 1, b: %{c: 2}}\n```","ref":"Moar.Map.html#deep_take/2"},{"type":"function","title":"Moar.Map.index_by/2","doc":"Converts a list of maps into a map of maps indexed by the values of one of the map keys.\nIt's a map-specific version of the more generic `Moar.Enum.index_by/2`.\n\n```elixir\niex> Moar.Map.index_by([%{name: \"Alice\", tid: \"alice\"}, %{name: \"Billy\", tid: \"billy\"}], :tid)\n%{\"alice\" => %{name: \"Alice\", tid: \"alice\"}, \"billy\" => %{name: \"Billy\", tid: \"billy\"}}\n```","ref":"Moar.Map.html#index_by/2"},{"type":"function","title":"Moar.Map.merge/2","doc":"Merges two enumerables into a single map. Supports `nil` values for either enumerable.\n\n```elixir\niex> Moar.Map.merge(%{a: 1}, [b: 2])\n%{a: 1, b: 2}\n\niex> Moar.Map.merge(nil, [a: 1])\n%{a: 1}\n```","ref":"Moar.Map.html#merge/2"},{"type":"function","title":"Moar.Map.merge_if_blank/2","doc":"Merges two maps, retaining any existing non-blank values.\n\n```elixir\niex> Moar.Map.merge_if_blank(%{a: 1, b: nil, c: \"\"}, %{a: 100, b: 2, c: 3, d: 4})\n%{a: 1, b: 2, c: 3, d: 4}\n```","ref":"Moar.Map.html#merge_if_blank/2"},{"type":"function","title":"Moar.Map.put_if_blank/3","doc":"Puts a key/value pair into the given map if the key is not alredy in the map, or if the value in the map is\nblank as defined by `Moar.Term.blank?/1`.\n\nAlso, the `map` parameter can be any enumerable that can be turned into a map via `Enum.into/2`.\n\n```elixir\niex> %{a: 1} |> Moar.Map.put_if_blank(:b, 2)\n%{a: 1, b: 2}\n\niex> %{a: 1, b: nil} |> Moar.Map.put_if_blank(:b, 2)\n%{a: 1, b: 2}\n\niex> %{a: 1, b: 3} |> Moar.Map.put_if_blank(:b, 2)\n%{a: 1, b: 3}\n```","ref":"Moar.Map.html#put_if_blank/3"},{"type":"function","title":"Moar.Map.put_new!/3","doc":"Like `Map.put_new/3` but raises if `key` already exists in `map`.\n\n```elixir\niex> Moar.Map.put_new!(%{a: 1}, :b, 2)\n%{a: 1, b: 2}\n\niex> Moar.Map.put_new!(%{a: 1, b: 2}, :b, 3)\n** (RuntimeError) Map already contains key: :b\n```","ref":"Moar.Map.html#put_new!/3"},{"type":"function","title":"Moar.Map.rename_key/2","doc":"Returns a copy of `map` with `old_key_name` changed to `new_key_name`.\n\n`old_key_name` and `new_key_name` are passed in as a `{old_key_name, new_key_name}` tuple.\n\n```elixir\niex> %{\"color\" => \"red\", \"size\" => \"medium\"} |> Moar.Map.rename_key({\"color\", \"colour\"})\n%{\"colour\" => \"red\", \"size\" => \"medium\"}\n```","ref":"Moar.Map.html#rename_key/2"},{"type":"function","title":"Moar.Map.rename_key/3","doc":"Returns a copy of `map` with `old_key_name` changed to `new_key_name`.\n\n```elixir\niex> %{\"color\" => \"red\", \"size\" => \"medium\"} |> Moar.Map.rename_key(\"color\", \"colour\")\n%{\"colour\" => \"red\", \"size\" => \"medium\"}\n```","ref":"Moar.Map.html#rename_key/3"},{"type":"function","title":"Moar.Map.rename_key!/2","doc":"Like `rename_key/2` but raises if `key` is not in `map`","ref":"Moar.Map.html#rename_key!/2"},{"type":"function","title":"Moar.Map.rename_key!/3","doc":"Like `rename_key/3` but raises if `key` is not in `map`","ref":"Moar.Map.html#rename_key!/3"},{"type":"function","title":"Moar.Map.rename_keys/2","doc":"Returns a copy of `map` after changing key names supplied by `keys_map`.\n\n```elixir\n%{\"behavior\" => \"chill\", \"color\" => \"red\"} |> Moar.Map.rename_keys(%{\"behavior\" => \"behaviour\", \"color\" => \"colour\"})\n%{\"behaviour\" => \"chill\", \"colour\" => \"red\"}\n```","ref":"Moar.Map.html#rename_keys/2"},{"type":"function","title":"Moar.Map.rename_keys!/2","doc":"Like `rename_keys/2` but raises if any key in `keys_map` is not in `map`.","ref":"Moar.Map.html#rename_keys!/2"},{"type":"function","title":"Moar.Map.stringify_keys/1","doc":"Converts keys in `map` to strings.\n\n```elixir\niex> Moar.Map.stringify_keys(%{a: 1, b: 2} )\n%{\"a\" => 1, \"b\" => 2}\n```","ref":"Moar.Map.html#stringify_keys/1"},{"type":"function","title":"Moar.Map.transform/3","doc":"Transforms values of `map` using `transformer` function.\n\n```elixir\niex> %{\"foo\" => \"chicken\", \"bar\" => \"cow\", \"baz\" => \"pig\"} |> Moar.Map.transform(\"foo\", &String.upcase/1)\n%{\"foo\" => \"CHICKEN\", \"bar\" => \"cow\", \"baz\" => \"pig\"}\n\niex> %{\"foo\" => \"chicken\", \"bar\" => \"cow\", \"baz\" => \"pig\"} |> Moar.Map.transform([\"foo\", \"bar\"], &String.upcase/1)\n%{\"foo\" => \"CHICKEN\", \"bar\" => \"COW\", \"baz\" => \"pig\"}\n```","ref":"Moar.Map.html#transform/3"},{"type":"function","title":"Moar.Map.validate_keys!/2","doc":"Validates that the keys of `map` are equal to or a subset of `valid_keys`. In addition to being a list, `valid_keys`\ncan be a map or struct, in which case the keys of the map or struct are used as the list of valid keys.\n\nIt returns the input map, or raises an exception if there are non-allowed keys.\n\n```elixir\niex> Moar.Map.validate_keys!(%{a: 1, b: 2}, [:a, :b, :c])\n%{a: 1, b: 2}\n\niex> Moar.Map.validate_keys!(%{a: 1, b: 2}, %{a: nil, b: 2, c: :foo})\n%{a: 1, b: 2}\n\niex> Moar.Map.validate_keys!(%{a: 1, b: 2}, [:a, :c])\n** (ArgumentError) Non-allowed keys found in map: [:b]\n```","ref":"Moar.Map.html#validate_keys!/2"},{"type":"module","title":"Moar.NaiveDateTime","doc":"NaiveDateTime-related functions. See also `Moar.DateTime`.","ref":"Moar.NaiveDateTime.html"},{"type":"function","title":"Moar.NaiveDateTime.add/2","doc":"Like `NaiveDateTime.add/2` but takes a `Moar.Duration`.\n\nSee also `subtract/1` and `Moar.DateTime.add/2`.\n\n> ##","ref":"Moar.NaiveDateTime.html#add/2"},{"type":"function","title":"Note {: .info} - Moar.NaiveDateTime.add/2","doc":">\n> This function is naive and intentionally doesn't account for real-world calendars and all of their complexity,\n> such as leap years, leap days, daylight saving time, past and future calendar oddities, etc.\n>\n> As [\"Falsehoods programmers believe about time\"](https://gist.github.com/timvisee/fcda9bbdff88d45cc9061606b4b923ca)\n> says, \"If you think you understand everything about time, you're probably doing it wrong.\"\n>\n> See [`Cldr.Calendar.plus/2`](https://hexdocs.pm/ex_cldr_calendars/Cldr.Calendar.html#plus/2) for one example\n> of a function that is far more likely to be correct.\n\n```elixir\niex> start = ~N[2022-01-01T00:00:00.000]\niex> Moar.NaiveDateTime.add(start, {3, :minute})\n~N[2022-01-01T00:03:00.000]\n```","ref":"Moar.NaiveDateTime.html#add/2-note-info"},{"type":"function","title":"Moar.NaiveDateTime.from_iso8601!/1","doc":"Like `NaiveDateTime.from_iso8601/1` but raises if the string cannot be parsed.\n\n```elixir\niex> Moar.NaiveDateTime.from_iso8601!(\"2022-01-01T00:00:00\")\n~N[2022-01-01T00:00:00]\n\niex> Moar.NaiveDateTime.from_iso8601!(\"2022-01-01T00:00:00+0800\")\n~N[2022-01-01T00:00:00]\n\niex> Moar.NaiveDateTime.from_iso8601!(\"Next Thursday after lunch\")\n** (ArgumentError) Invalid ISO8601 format: \"Next Thursday after lunch\"\n```","ref":"Moar.NaiveDateTime.html#from_iso8601!/1"},{"type":"function","title":"Moar.NaiveDateTime.subtract/2","doc":"Subtracts `duration` from `date_time`.\n\nSee also `add/1` and `Moar.DateTime.subtract/2`.\n\n> ##","ref":"Moar.NaiveDateTime.html#subtract/2"},{"type":"function","title":"Note {: .info} - Moar.NaiveDateTime.subtract/2","doc":">\n> This function is naive and intentionally doesn't account for real-world calendars and all of their complexity,\n> such as leap years, leap days, daylight saving time, past and future calendar oddities, etc.\n>\n> As [\"Falsehoods programmers believe about time\"](https://gist.github.com/timvisee/fcda9bbdff88d45cc9061606b4b923ca)\n> says, \"If you think you understand everything about time, you're probably doing it wrong.\"\n>\n> See [`Cldr.Calendar.minus/4`](https://hexdocs.pm/ex_cldr_calendars/Cldr.Calendar.html#minus/4) for one example\n> of a function that is far more likely to be correct.\n\n```elixir\niex> start = ~N[2022-01-01T00:03:00.000]\niex> Moar.NaiveDateTime.subtract(start, {3, :minute})\n~N[2022-01-01T00:00:00.000]\n```","ref":"Moar.NaiveDateTime.html#subtract/2-note-info"},{"type":"function","title":"Moar.NaiveDateTime.to_iso8601_rounded/1","doc":"Like `NaiveDateTime.to_iso8601/1` but rounds to the nearest second first.\n\n```elixir\niex> Moar.NaiveDateTime.to_iso8601_rounded(~N[2022-01-01T01:02:03.456789])\n\"2022-01-01T01:02:03\"\n```","ref":"Moar.NaiveDateTime.html#to_iso8601_rounded/1"},{"type":"function","title":"Moar.NaiveDateTime.utc_now/1","doc":"Returns the current UTC time plus or minus the given duration.\n\n```elixir\niex> Moar.NaiveDateTime.utc_now(plus: {10500, :millisecond})\n...> |> Moar.Duration.format([:approx, :from_now])\n\"10 seconds from now\"\n\niex> Moar.NaiveDateTime.utc_now(minus: {10, :second})\n...> |> Moar.Duration.format([:approx, :ago])\n\"10 seconds ago\"\n```","ref":"Moar.NaiveDateTime.html#utc_now/1"},{"type":"module","title":"Moar.Opts","doc":"Extracts keys and values from enumerables, especially from function options.\n\nThere are two main functions, each of which takes an opts enumerable as input. `get/3` extracts\none value from the opts with an optional default value. `take/2` extracts multiple values from the opts\nwith optional default values for some or all keys.\n\n`get/3` and `take/2` differ from their `Map` and `Keyword` counterparts in the following ways:\n* `get/3` and `take/2` accept any enumerable, including maps and keyword lists.\n* `get/3` and `take/2` will fall back to the default value if the given key's value is blank\n  as defined by `Moar.Term.blank?/1` (`nil`, empty strings, strings made up only of whitespace,\n  empty lists, and empty maps). The corresponding `Map` and `Keyword` functions only fall back to\n  the default value if the value is exactly `nil`.\n* `take/2` allows default values to be specified.\n* `take/2` will return the value for a requested key even if the key is not in the input enumerable.\n\nExample using `get/2` and `get/3`:\n\n```elixir\ndef build_url(path, opts \\\\ []) do\n  %URI{\n    path: path,\n    host: Moar.Opts.get(opts, :host, \"localhost\"),\n    port: Moar.Opts.get(opts, :port),\n    scheme: \"https\"\n  } |> URI.to_string()\nend\n```\n\nExamples using `take/2`:\n\n```elixir\n# example using pattern matching\ndef build_url(path, opts \\ []) do\n  %{host: h, port: p} = Moar.Opts.take(opts, [:port, host: \"localhost\"])\n  %URI{path: path, host: h, port: p, scheme: \"https\"} |> URI.to_string()\nend\n\n# example rebinding `opts` to the parsed opts\ndef build_url(path, opts \\ []) do\n  opts = Moar.Opts.take(opts, [:port, host: \"localhost\"])\n  %URI{path: path, host: opts.host, port: opts.port, scheme: \"https\"} |> URI.to_string()\nend\n```","ref":"Moar.Opts.html"},{"type":"function","title":"Moar.Opts.get/3","doc":"Get the value of `key` from `input`, falling back to optional `default` if the key does not exist,\nor if its value is blank (via `Moar.Term.blank?/1`).\n\n```elixir\niex> [a: 1, b: 2] |> Moar.Opts.get(:a)\n1\n\niex> [a: 1, b: 2] |> Moar.Opts.get(:c)\nnil\n\niex> [a: 1, b: 2, c: \"\"] |> Moar.Opts.get(:c)\nnil\n\niex> [a: 1, b: 2, c: %{}] |> Moar.Opts.get(:c)\nnil\n\niex> [a: 1, b: 2, c: \"   \"] |> Moar.Opts.get(:c, 300)\n300\n```","ref":"Moar.Opts.html#get/3"},{"type":"function","title":"Moar.Opts.take/2","doc":"Get the value each key in `keys` from `input`, falling back to optional default values for keys that\ndo not exist, or for values that are blank (via `Moar.Term.blank?/1`).\n\nIf `key` does not exist in `keys`, return `nil`, or return the default value if provided.\n\n`keys` is a list of keys (e.g., `[:a, :b]`),\na keyword list of keys and default values (e.g., `[a: 1, b: 2]`),\nor a hybrid list/keyword list (e.g., `[:a, b: 2]`)\n\n```elixir\niex> [a: 1, b: 2] |> Moar.Opts.take([:a, :c])\n%{a: 1, c: nil}\n\niex> [a: 1, b: 2] |> Moar.Opts.take([:a, b: 0, c: 3])\n%{a: 1, b: 2, c: 3}\n```","ref":"Moar.Opts.html#take/2"},{"type":"module","title":"Moar.Protocol","doc":"Protocol-related functions.","ref":"Moar.Protocol.html"},{"type":"function","title":"Moar.Protocol.implements!/2","doc":"Returns `x` or raises if `x` does not implment `protocol`.\n\n```elixir\niex> Moar.Protocol.implements!(~D[2000-01-02], String.Chars)\n~D[2000-01-02]\n```","ref":"Moar.Protocol.html#implements!/2"},{"type":"function","title":"Moar.Protocol.implements?/2","doc":"Returns true if `x` implements `protocol`.\n\n```elixir\niex> Moar.Protocol.implements?(~D[2000-01-02], String.Chars)\ntrue\n```","ref":"Moar.Protocol.html#implements?/2"},{"type":"module","title":"Moar.Random","doc":"Generates random data.","ref":"Moar.Random.html"},{"type":"function","title":"Moar.Random.float/2","doc":"Return a random float greater than or equal to `min` and less than `max`","ref":"Moar.Random.html#float/2"},{"type":"function","title":"Moar.Random.integer/1","doc":"Returns a random integer between `0` and `max`.","ref":"Moar.Random.html#integer/1"},{"type":"function","title":"Moar.Random.string/1","doc":"Returns a base64- or base32-encoded random string of 32 characters.\nSee `Moar.Random.string/2`.","ref":"Moar.Random.html#string/1"},{"type":"function","title":"Moar.Random.string/2","doc":"Returns a base64- or base32-encoded random string of given length.\n\n```elixir\niex> Moar.Random.string()\n\"Sr/y4m/YiVSJcIgI5lG+76vMfaZ7KZ7c\"\niex> Moar.Random.string(5)\n\"9pJrK\"\niex> Moar.Random.string(5, :base32)\n\"AC53Z\"\n```","ref":"Moar.Random.html#string/2"},{"type":"type","title":"Moar.Random.encoding/0","doc":"","ref":"Moar.Random.html#t:encoding/0"},{"type":"module","title":"Moar.Regex","doc":"Regex-related functions.","ref":"Moar.Regex.html"},{"type":"function","title":"Moar.Regex.named_capture/3","doc":"Returns a single named capture. Can take the first two args in any order.\n\n```\niex> Moar.Regex.named_capture(~r/a(? b)c(? d)/, \"abcd\", \"foo\")\n\"b\"\n\niex> Moar.Regex.named_capture(\"abcd\", ~r/a(? b)c(? d)/, \"foo\")\n\"b\"\n```","ref":"Moar.Regex.html#named_capture/3"},{"type":"function","title":"Moar.Regex.named_captures/3","doc":"Like `Regex.named_captures/3` but can take the first two args in any order.\n\n```\niex> Moar.Regex.named_captures(~r/a(? b)c(? d)/, \"abcd\")\n%{\"bar\" => \"d\", \"foo\" => \"b\"}\n\niex> Moar.Regex.named_captures(\"abcd\", ~r/a(? b)c(? d)/)\n%{\"bar\" => \"d\", \"foo\" => \"b\"}\n```","ref":"Moar.Regex.html#named_captures/3"},{"type":"module","title":"Moar.Retry","doc":"Retryable functions.\n\nThese functions are particularly useful in higher-level tests, though they can be used for any\nkind of waiting.\n\nSee also the [retry](https://hex.pm/packages/retry) Hex package. \n\nOne use of `rescue_for/3` in a test is to wait until an assertion succeeds:\n\n```elixir\n# wait up to 5 seconds for the element with ID \"status\" to be \"finished\"\nMoar.Retry.rescue_for!(5000, fn ->\n  assert view |> element(\"#status\") |> render() == \"finished\"\nend)\n```\n\nOne use of `retry_for/3` in a test is to wait until something changes before making an assertion:\n\n```elixir\nMoar.Retry.retry_for(5000, fn ->\n  view |> element(\"#status\") |> render() == \"finished\"\nend)\n\nassert view |> element(\"#total\") |> render() == \"8,675,309\"\n```","ref":"Moar.Retry.html"},{"type":"function","title":"Moar.Retry.rescue_for!/3","doc":"Run `fun` every `interval_ms` until it doesn't raise an exception.\nIf `timeout` expires, the exception will be re-raised.\n\n* `timeout` can be an integer in milliseconds or a `Moar.Duration` tuple\n\n```elixir\niex> Moar.Retry.rescue_for!(20, fn -> raise \"always fails\" end, 2)\n** (RuntimeError) always fails\n\niex> Moar.Retry.rescue_for!({20, :millisecond}, fn -> raise \"always fails\" end, 2)\n** (RuntimeError) always fails\n```","ref":"Moar.Retry.html#rescue_for!/3"},{"type":"function","title":"Moar.Retry.rescue_until!/3","doc":"Run `fun` every `interval_ms` until it doesn't raise an exception.\nIf the current time reaches `expiry`, the exception will be re-raised.\n\n```elixir\niex> date_time = DateTime.add(DateTime.utc_now(), 20, :millisecond)\niex> Moar.Retry.rescue_until!(date_time, fn -> raise \"always fails\" end, 2)\n** (RuntimeError) always fails\n```","ref":"Moar.Retry.html#rescue_until!/3"},{"type":"function","title":"Moar.Retry.retry_for/3","doc":"Run `fun` every `interval_ms` until it returns a truthy value, returning `{:ok,  }`.\nIf `timeout` expires, returns `{:error, :timeout}`.\n\n* `timeout` can be an integer in milliseconds or a `Moar.Duration` tuple\n\n```elixir\niex> Moar.Retry.retry_for(20, fn -> 10 end, 2)\n{:ok, 10}\n\niex> Moar.Retry.retry_for(20, fn -> false end, 2)\n{:error, :timeout}\n```","ref":"Moar.Retry.html#retry_for/3"},{"type":"function","title":"Moar.Retry.retry_until/3","doc":"Run `fun` every `interval_ms` until it returns a truthy value, returning `{:ok,  }`.\nIf the current time reaches `expiry`, returns `{:error, :timeout}`.\n\n```elixir\niex> date_time = DateTime.add(DateTime.utc_now(), 20, :millisecond)\niex> Moar.Retry.retry_until(date_time, fn -> false end, 2)\n{:error, :timeout}\n```","ref":"Moar.Retry.html#retry_until/3"},{"type":"module","title":"Moar.String","doc":"String-related functions.","ref":"Moar.String.html"},{"type":"function","title":"Moar.String.append_unless_blank/2","doc":"Appends `suffix` to `string` unless `string` is blank according to `Moar.Term.blank?/1`.\n\n```elixir\niex> Moar.String.append_unless_blank(\"foo\", \"-bar\")\n\"foo-bar\"\n\niex> Moar.String.append_unless_blank(\"\", \"-bar\")\n\"\"\n\niex> Moar.String.append_unless_blank(nil, \"-bar\")\nnil\n```","ref":"Moar.String.html#append_unless_blank/2"},{"type":"function","title":"Moar.String.compare/3","doc":"Compares two strings, returning `:lt`, `:eq`, or `:gt` depending on whether the first arg is less than, equal to, or\ngreater than the second arg. Accepts one or more functions that transform the inputs before comparison.\n\nSee `Moar.String.compare?/2` for a version that returns `true` or `false`.\n\n```\niex> Moar.String.compare(\"foo\", \"FOO\")\n:gt\n\niex> Moar.String.compare(\"foo\", \"FOO\", &String.downcase/1)\n:eq\n\niex> Moar.String.compare(\"foo bar\", \" FOO    bar \", [&String.downcase/1, &Moar.String.squish/1])\n:eq\n```","ref":"Moar.String.html#compare/3"},{"type":"function","title":"Moar.String.compare?/3","doc":"Compares two strings, returning `true` if the first arg is less than or equal to the second arg, or `false` if the\nfirst arg is greater than the second arg. Accepts one or more functions that transform the inputs before comparison.\nUseful for sorter functions like what is passed into `Enum.sort/2`.\n\nSee `Moar.String.compare/2` for a version that returns `:lt`, `:eq`, or `:gt`.\n\n```\niex> Moar.String.compare?(\"foo\", \"FOO\")\nfalse\n\niex> Moar.String.compare?(\"foo\", \"FOO\", &String.downcase/1)\ntrue\n\niex> Moar.String.compare?(\"foo bar\", \" FOO    bar \", [&String.downcase/1, &Moar.String.squish/1])\ntrue\n```","ref":"Moar.String.html#compare?/3"},{"type":"function","title":"Moar.String.count_leading_spaces/1","doc":"Returns the number of leading spaces. Only considers \"plain\" spaces, not all unicode whitespace.\n\n```elixir\niex> Moar.String.count_leading_spaces(\"  foo\")\n2\n```","ref":"Moar.String.html#count_leading_spaces/1"},{"type":"function","title":"Moar.String.dasherize/1","doc":"Dasherizes `term`. A shortcut to `slug(term, \"-\")`.\n\nSee docs for `slug/2`.\n\n```elixir\niex> Moar.String.dasherize(\"foo bar\")\n\"foo-bar\"\n```","ref":"Moar.String.html#dasherize/1"},{"type":"function","title":"Moar.String.inner_truncate/3","doc":"Truncate `s` to `max_length` by replacing the middle of the string with `replacement`, which defaults to\nthe single unicode character `…`.\n\nNote that the final length of the string will be `max_length` plus the length of `replacement`.\n\n```elixir\niex> Moar.String.inner_truncate(\"abcdefghijklmnopqrstuvwxyz\", 10)\n\"abcde…vwxyz\"\n\niex> Moar.String.inner_truncate(\"abcdefghijklmnopqrstuvwxyz\", 10, \"<==>\")\n\"abcde<==>vwxyz\"\n```","ref":"Moar.String.html#inner_truncate/3"},{"type":"function","title":"Moar.String.pluralize/3","doc":"Pluralizes a string.\n\nWhen `count` is -1 or 1, returns the second argument (the singular string).\n\nOtherwise, returns the third argument (the pluralized string), or if the third argument is a function,\ncalls the function with the singular string as an argument.\n\n```elixir\niex> Moar.String.pluralize(1, \"fish\", \"fishies\")\n\"fish\"\n\niex> Moar.String.pluralize(2, \"fish\", \"fishies\")\n\"fishies\"\n\niex> Moar.String.pluralize(2, \"fish\", fn singular -> singular <> \"ies\" end)\n\"fishies\"\n\niex> Moar.String.pluralize(2, \"fish\", &(&1 <> \"ies\"))\n\"fishies\"\n```","ref":"Moar.String.html#pluralize/3"},{"type":"function","title":"Moar.String.remove_marked_whitespace/1","doc":"Removes all whitespace following a backspace+v escape code.\n\nEspecially useful in test assertions where only some of the whitespace matters.\n```\niex> Moar.String.remove_marked_whitespace(\"one two three\u000B\t   four five\")\n\"one two threefour five\"\n```","ref":"Moar.String.html#remove_marked_whitespace/1"},{"type":"function","title":"Moar.String.secure_compare/2","doc":"Compares the two binaries in constant-time to avoid timing attacks.\nSee:  .\n\n```elixir\niex> Moar.String.secure_compare(\"foo\", \"bar\")\nfalse\n```","ref":"Moar.String.html#secure_compare/2"},{"type":"function","title":"Moar.String.slug/2","doc":"Creates slugs like `foo-bar-123` or `foo_bar` from various input types.\n\nConverts strings, atoms, and anything else that implements `String.Chars`, plus lists of those things,\nto a single string after removing non-alphanumeric characters, and then joins them with `joiner`.\nExisting occurrences of `joiner` are kept, including leading and trailing ones.\n\n`dasherize/1` and `underscore/1` are shortcuts that specify a joiner.\n\n```elixir\niex> Moar.String.slug(\"foo bar\", \"_\")\n\"foo_bar\"\n\niex> Moar.String.slug(\"foo bar\", \"+\")\n\"foo+bar\"\n\niex> Moar.String.slug([\"foo\", \"bar\"], \"+\")\n\"foo+bar\"\n\niex> Moar.String.slug(\"_foo bar\", \"_\")\n\"_foo_bar\"\n\niex> [\"foo\", \"FOO\", :foo] |> Enum.map(&Moar.String.slug(&1, \"-\"))\n[\"foo\", \"foo\", \"foo\"]\n\niex> [\"foo-bar\", \"foo_bar\", :foo_bar, \" fooBar \", \"  ?foo ! bar  \"] |> Enum.map(&Moar.String.slug(&1, \"-\"))\n[\"foo-bar\", \"foo-bar\", \"foo-bar\", \"foo-bar\", \"foo-bar\"]\n```","ref":"Moar.String.html#slug/2"},{"type":"function","title":"Moar.String.squish/1","doc":"Trims spaces from the beginning and end of a string, and replaces consecutive whitespace characters with a single\nspace.\n\n```elixir\niex> Moar.String.squish(\"  foo   bar  \tbaz \")\n\"foo bar baz\"\n```","ref":"Moar.String.html#squish/1"},{"type":"function","title":"Moar.String.surround/2","doc":"Adds `surrounder` to the beginning and end of `s`.\n\n```elixir\niex> Moar.String.surround(\"Hello\", \"**\")\n\"**Hello**\"\n```","ref":"Moar.String.html#surround/2"},{"type":"function","title":"Moar.String.surround/3","doc":"Adds `prefix` to the beginning of `s` and `suffix` to the end.\n\n```elixir\niex> Moar.String.surround(\"Hello\", \"“\", \"”\")\n\"“Hello”\"\n```","ref":"Moar.String.html#surround/3"},{"type":"function","title":"Moar.String.to_case/2","doc":"Change the case of a string.\n\n```elixir\niex> Moar.String.to_case(\"text_with_case\", :camel_case)\n\"TextWithCase\"\niex> Moar.String.to_case(\"textWithCase\", :camel_case)\n\"TextWithCase\"\niex> Moar.String.to_case(\"some random text\", :camel_case)\n\"SomeRandomText\"\n\niex> Moar.String.to_case(\"text_with_case\", :lower_camel_case)\n\"textWithCase\"\niex> Moar.String.to_case(\"textWithCase\", :lower_camel_case)\n\"textWithCase\"\niex> Moar.String.to_case(\"some random text\", :lower_camel_case)\n\"someRandomText\"\n\niex> Moar.String.to_case(\"text_with_case\", :snake_case)\n\"text_with_case\"\niex> Moar.String.to_case(\"textWithCase\", :snake_case)\n\"text_with_case\"\niex> Moar.String.to_case(\"some random text\", :snake_case)\n\"some_random_text\"\n```","ref":"Moar.String.html#to_case/2"},{"type":"function","title":"Moar.String.to_integer/1","doc":"Converts a string to an integer. Returns `nil` if the argument is `nil` or empty string.\nReturns the argument without complaint if it is already an integer.\n\n```elixir\niex> Moar.String.to_integer(\"12,345\")\n12_345\n\niex> Moar.String.to_integer(\"\")\nnil\n\niex> Moar.String.to_integer(12_345)\n12_345\n```","ref":"Moar.String.html#to_integer/1"},{"type":"function","title":"Moar.String.to_integer/2","doc":"Like `to_integer/1` but with options:\n* `:lenient` option removes non-digit characters first\n* `default:` option specifies a default in case `s` is nil\n\n```elixir\niex> Moar.String.to_integer(\"USD$25\", :lenient)\n25\n\niex> Moar.String.to_integer(nil, default: 0)\n0\n```","ref":"Moar.String.html#to_integer/2"},{"type":"function","title":"Moar.String.trim/1","doc":"Like `String.trim/1` but returns `nil` if the argument is nil.","ref":"Moar.String.html#trim/1"},{"type":"function","title":"Moar.String.truncate_at/3","doc":"Truncates `s` at the last instance of `at`, causing the string to be at most `limit` characters.\n\n```elixir\niex> Moar.String.truncate_at(\"I like apples. I like bananas. I like cherries.\", \".\", 35)\n\"I like apples. I like bananas.\"\n```","ref":"Moar.String.html#truncate_at/3"},{"type":"function","title":"Moar.String.underscore/1","doc":"Underscores `term`. A shortcut to `slug(term, \"_\")`.\n\nSee docs for `slug/2`.\n\n```elixir\niex> Moar.String.underscore(\"foo bar\")\n\"foo_bar\"\n```","ref":"Moar.String.html#underscore/1"},{"type":"function","title":"Moar.String.unindent/1","doc":"Unindents a string by finding the smallest indentation of the string, and removing that many spaces from each line.\nOnly considers \"plain\" spaces, not all unicode whitespace.\n\n```elixir\niex> \"\"\"\n...>      ant\n...>    bat\n...>      cat\n...>        dog\n...> \"\"\" |> Moar.String.unindent()\n\"  ant\nbat\n  cat\n    dog\n\"\n```","ref":"Moar.String.html#unindent/1"},{"type":"function","title":"Moar.String.unindent/2","doc":"Unindents a string by `count` spaces. Only considers \"plain\" spaces, not all unicode whitespace.\n\n```elixir\niex> \"\"\"\n...> foo\n...>   bar\n...>     baz\n...> \"\"\" |> Moar.String.unindent(2)\n\"foo\nbar\n  baz\n\"\n```","ref":"Moar.String.html#unindent/2"},{"type":"type","title":"Moar.String.string_case/0","doc":"","ref":"Moar.String.html#t:string_case/0"},{"type":"module","title":"Moar.Sugar","doc":"Syntactic sugar functions.\n\nThese functions are intended to be used by importing the functions or the whole module:\n\n```\nimport Moar.Sugar, only: [noreply: 1]\n\ndef handle_event(\"foo\", _params, socket) do\n  socket |> assign(foo: \"bar\") |> noreply()\nend\n```","ref":"Moar.Sugar.html"},{"type":"function","title":"Moar.Sugar.error/1","doc":"Wraps a term in an :error tuple. Useful in pipelines.\n\n```elixir\niex> %{} |> Map.put(:count, \"unknown\") |> Moar.Sugar.error()\n{:error, %{count: \"unknown\"}}\n```","ref":"Moar.Sugar.html#error/1"},{"type":"function","title":"Moar.Sugar.error!/1","doc":"Unwraps an :error tuple, raising if the term is not an :error tuple.\n\n```elixir\niex> {:error, 1} |> Moar.Sugar.error!()\n1\n```","ref":"Moar.Sugar.html#error!/1"},{"type":"function","title":"Moar.Sugar.noreply/1","doc":"Wraps a term in a :noreply tuple. Useful in pipelines.\n\n```elixir\niex> %{} |> Map.put(:count, 0) |> Moar.Sugar.noreply()\n{:noreply, %{count: 0}}\n```","ref":"Moar.Sugar.html#noreply/1"},{"type":"function","title":"Moar.Sugar.ok/1","doc":"Wraps a term in an :ok tuple. Useful in pipelines.\n\n```elixir\niex> %{} |> Map.put(:count, 10) |> Moar.Sugar.ok()\n{:ok, %{count: 10}}\n```","ref":"Moar.Sugar.html#ok/1"},{"type":"function","title":"Moar.Sugar.ok!/1","doc":"Unwraps an :ok tuple, raising if the term is not an :ok tuple.\n\n```elixir\niex> {:ok, 1} |> Moar.Sugar.ok!()\n1\n```","ref":"Moar.Sugar.html#ok!/1"},{"type":"function","title":"Moar.Sugar.returning/2","doc":"Accepts two arguments and returns the second.\n\nUseful at the end of the pipeline when you want to return a different value than the last result of the pipeline,\nsuch as when the pipeline has side effects.\n\n```elixir\niex> %{} |> Map.put(:count, 20) |> Moar.Sugar.returning(:count_updated)\n:count_updated\n```","ref":"Moar.Sugar.html#returning/2"},{"type":"module","title":"Moar.Term","doc":"Blank/present functions for terms.\n\nA term is considered present when it is not blank.\n\nA term is considered blank when:\n\n* it is `nil`\n* it is `false`\n* it is a string, and its length after being trimmed is 0\n* it is an empty list\n* it is an empty map","ref":"Moar.Term.html"},{"type":"function","title":"Moar.Term.blank?/1","doc":"Returns true if the term is blank, nil, or empty.\n\n```elixir\niex> Moar.Term.blank?(nil)\ntrue\n\niex> Moar.Term.blank?(\"   \")\ntrue\n\niex> Moar.Term.blank?([])\ntrue\n\niex> Moar.Term.blank?(%{})\ntrue\n```","ref":"Moar.Term.html#blank?/1"},{"type":"function","title":"Moar.Term.presence/2","doc":"Returns the value if it is present (via `present?`), or else returns the default value.\n\n```elixir\niex> Moar.Term.presence(20, 100)\n20\n\niex> Moar.Term.presence(nil, 100)\n100\n```","ref":"Moar.Term.html#presence/2"},{"type":"function","title":"Moar.Term.present?/1","doc":"Returns true if the term is not blank, nil, or empty.\n\n```elixir\niex> Moar.Term.present?(1)\ntrue\n\niex> Moar.Term.present?([1])\ntrue\n\niex> Moar.Term.present?(%{a: 1})\ntrue\n\niex> Moar.Term.present?(\"1\")\ntrue\n```","ref":"Moar.Term.html#present?/1"},{"type":"module","title":"Moar.Tuple","doc":"Tuple-related functions.","ref":"Moar.Tuple.html"},{"type":"function","title":"Moar.Tuple.from_list!/1","doc":"Converts a list of tuples to a single tuple whose first element is the first element of each tuple in\nthe list (which must all be the same), and whose second element is a list containing the second elements\nof each tuple in the list.\n\nRaises if the list contains tuples whose first elements are not all the same.\n\n```elixir\niex> Moar.Tuple.from_list!([{:ok, :a}, {:ok, :b}])\n{:ok, [:a, :b]}\n\niex> Moar.Tuple.from_list!([{:a, 1}, {:a, 2}, {:a, 3}])\n{:a, [1, 2, 3]}\n\niex> Moar.Tuple.from_list!([{:a, 1}, {:b, 2}, {:a, 3}])\n** (RuntimeError) Expected all items in the list to have have the same first element, but got: [:a, :b]\n```","ref":"Moar.Tuple.html#from_list!/1"},{"type":"function","title":"Moar.Tuple.reduce/1","doc":"Reduces a list of tuples to map where values are consolidated by the first element of each input tuple.\n\n```elixir\niex> Moar.Tuple.reduce([{:ok, 1}, {:ok, 2}])\n%{ok: [1, 2]}\n\niex> Moar.Tuple.reduce([{:ok, 1}, {:ok, 2}, {:error, 3}, {:ok, 4}])\n%{ok: [1, 2, 4], error: [3]}\n```","ref":"Moar.Tuple.html#reduce/1"},{"type":"module","title":"Moar.URI","doc":"URI/URL-related functions.","ref":"Moar.URI.html"},{"type":"function","title":"Moar.URI.fix/1","doc":"Applies some fixes to a URI string if necessary. Returns `nil` when given `nil`.\n\nAdds a default path if one is not given:\n\n```elixir\niex> Moar.URI.fix(\"https://www.example.com\")\n\"https://www.example.com/\"\n\niex> Moar.URI.fix(\"https://www.example.com/\")\n\"https://www.example.com/\"\n```\n\nAdds an `https` scheme if no scheme is given:\n\n```elixir\niex> Moar.URI.fix(\"www.example.com/\")\n\"https://www.example.com/\"\n\niex> Moar.URI.fix(\"http://www.example.com/\")\n\"http://www.example.com/\"\n```\n\nLowercases the scheme:\n\n```elixir\niex> Moar.URI.fix(\"HttpS://www.example.com/\")\n\"https://www.example.com/\"\n```\n\nUnwraps CDATA:\n\n```elixir\niex> Moar.URI.fix(\"<![CDATA[http://example.com/]]>\")\n\"http://example.com/\"\n```","ref":"Moar.URI.html#fix/1"},{"type":"function","title":"Moar.URI.format/2","doc":"Formats a URI as a string with the given format. Format options:\n\n`scheme_host_port`: converts to ` :// [: ]` format, where the port is only rendered\nif the original URI includes the port.\n\n`scheme_host_port_path`: converts to ` :// [: ]/ ` format, where the port is only rendered\nif the original URI included the port, and the path is rendered as `/` if the original URI does not include a path.\n\n`simple_string`: converts to a simplified string representation of a URI for display purposes. Scheme, port, params,\nand fragments are removed. `nil` is converted to an empty string.\n\n```elixir\niex> url = \"https://www.example.com:446/crackers/potato%20chips/fruit?a=1&b=2#something\"\niex> Moar.URI.format(url, :scheme_host_port)\n\"https://www.example.com:446\"\n\niex> url = \"https://www.example.com:446/crackers/potato%20chips/fruit?a=1&b=2#something\"\niex> Moar.URI.format(url, :scheme_host_port_path)\n\"https://www.example.com:446/crackers/potato%20chips/fruit\"\n\niex> url = \"https://www.example.com:446/crackers/potato%20chips/fruit?a=1&b=2#something\"\niex> Moar.URI.format(url, :simple_string)\n\"www.example.com/crackers/potato chips/fruit\"\n\niex> Moar.URI.format(nil, :simple_string)\n\"\"\n```","ref":"Moar.URI.html#format/2"},{"type":"function","title":"Moar.URI.to_simple_string/1","doc":"Returns a simplified string representation of a URI for display purposes. Scheme, port, params, and fragments are\nremoved. `nil` is converted to an empty string.","ref":"Moar.URI.html#to_simple_string/1"},{"type":"function","title":"Moar.URI.valid?/1","doc":"Returns `true` if the URI has a host and scheme, and if it has a path, the path does not contain spaces.\n\n```elixir\niex> Moar.URI.valid?(%URI{host: \"example.com\", path: \"users/1\", scheme: \"https\"})\ntrue\n\niex> Moar.URI.valid?(%URI{host: \"example.com\", path: \"users/1\", scheme: nil})\nfalse\n\niex> Moar.URI.valid?(%URI{host: \"example.com\", path: \"spaces not allowed\", scheme: \"https\"})\nfalse\n```","ref":"Moar.URI.html#valid?/1"},{"type":"function","title":"Moar.URI.web_url?/1","doc":"Returns `true` when given a valid URI string with an `http` or `https` scheme.\n\n```elixir\niex> Moar.URI.web_url?(nil)\nfalse\n\niex> Moar.URI.web_url?(\"ftp://example.org\")\nfalse\n\niex> Moar.URI.web_url?(\"http://example.org\")\ntrue\n```","ref":"Moar.URI.html#web_url?/1"},{"type":"module","title":"Moar.Version","doc":"Version-related functions.","ref":"Moar.Version.html"},{"type":"function","title":"Moar.Version.compare/2","doc":"Like `Version.compare/2` but first normalizes versions via `normalize/1`.\n\n```\niex> Elixir.Version.compare(\"1.2\", \"1.2.3\")\n** (Version.InvalidVersionError) invalid version: \"1.2\"\n\niex> Moar.Version.compare(\"1.2\", \"1.2.3\")\n:lt\n```","ref":"Moar.Version.html#compare/2"},{"type":"function","title":"Moar.Version.normalize/1","doc":"Adds major, minor, and patch versions if needed to create a string with major, minor, and patch numbers.\nDoes not support versions that have anything other than numbers (like `1.2.3-beta4`).\n\n```\niex> Moar.Version.normalize(\"1.2\")\n\"1.2.0\"\n```","ref":"Moar.Version.html#normalize/1"},{"type":"extras","title":"Moar","doc":"# Moar\n\n[![CI](https://github.com/synchronal/moar/actions/workflows/tests.yml/badge.svg)](https://github.com/synchronal/moar/actions)\n[![Hex pm](http://img.shields.io/hexpm/v/moar.svg?style=flat)](https://hex.pm/packages/moar) [![License](http://img.shields.io/github/license/synchronal/moar.svg?style=flat)](https://github.com/synchronal/moar/blob/main/LICENSE.md)\n\nAn assortment of useful functions.\n\nThe docs can be found at https://hexdocs.pm/moar","ref":"readme.html"},{"type":"extras","title":"Installation - Moar","doc":"The package can be installed by adding `moar` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:moar, \"~> 1.49\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Similar libraries - Moar","doc":"- [Bunch](https://hexdocs.pm/bunch/readme.html)\n- [Swiss](https://hexdocs.pm/swiss/readme.html)","ref":"readme.html#similar-libraries"},{"type":"extras","title":"A quick tour - Moar","doc":"`Moar.Assertions`\n\n- `assert_contains` asserts that a list or map contains one or more elements.\n- `assert_eq` is a pipeable equality assertion, with options such as the ability to ignore order when comparing lists,\n  ignore space when comparing strings, consider only certain keys of a map, returning a different value than what was\n  passed in, and asserting that a value is within some delta (which can be a number or time duration).\n- `assert_recent` asserts that a datetime is pretty close to now.\n- `assert_that` asserts that a pre-condition and post-condition are true after performing an action.\n- `refute_that` asserts that a condition didn't change after performing an action.\n\n`Moar.Atom`\n\n- `atomize` when given an atom, returns it; when given a string, converts it to an atom, replacing non-alphanumeric\n  characters with underscores.\n- `from_string` and `to_string` convert between strings and atoms, and don't fail if you try to convert an atom to an atom\n  or a string to a string.\n- `to_existing_atom` is like `String.to_existing_atom/1` but can accept an atom as a param.\n\n`Moar.DateTime` and `Moar.NaiveDateTime`\n\n- `add` and `subtract` can add and subtract a `Moar.Duration`, which is a tuple with a time unit, like `{27, :minute}`.\n- `between?` determines whether a datetime is between two other datetimes.\n- `from_iso8601!` raises if the string is not in ISO 8601 format.\n- `recent?` returns true if the given datetime was at most one minute ago.\n- `to_iso8601_rounded` converts to an ISO 8601 string, truncated to the second.\n- `utc_now` takes `plus` and `minus` options to get the current time plus or minus some duration.\n- `within?` determines whether a given datetime is within the given duration.\n\n`Moar.Difference`\n\n- a protocol that defines `diff(a, b)` along with implementations for datetimes.\n\n`Moar.Duration`\n\n- is a `{time, unit}` tuple (like `{27, :minute}`) and supports the regular `t:System.time_unit/0` values and also\n  `:minute`, `:hour`, and `:day`.\n- `ago` returns the duration between a given datetime and now.\n- `approx` shifts the duration to a simple approximate value.\n- `between` returns the duration between two datetimes.\n- `convert` converts a duration to a new time unit, returning only the value.\n- `format` formats a duration in long (`\"3 seconds\"`) or short (`\"3s\"`) format with optional transformers and suffix.\n- `from_now` returns a duration between now and a given datetime.\n- `humanize` converts the duration to the highest possible time unit.\n- `shift`, `shift_up`, and `shift_down` convert a duration to a new time unit.\n- `to_string` renders a duration into a string like `\"27 minutes\"`.\n\n`Moar.Enum`\n\n- `at!` is like `Enum.at` but raises if the index is out of bounds.\n- `compact` rejects nil values.\n- `find_indices` returns the indices of matching elements\n- `find_indices!` returns the indices of matching elements and raises if any element is not found.\n- `first!` returns the first item or raises if there isn't a first item.\n- `index_by` converts an enum into a list of maps indexed by the given function.\n- `into!` is like `Enum.into` but allows `nil` as its first argument.\n- `is_map_or_keyword` returns true if the value is a map or a keyword list (unfortunately cannot be used as a guard).\n- `isort` and `isort_by` sort case-insensitively.\n- `lists_to_maps` converts a list of lists to a lists of maps using the provided list of keys.\n- `take_at` returns a list of elements at the given indices.\n- `test_ids` is like `tids` (see below) with a slightly different spelling.\n- `tids` extracts `tid` fields. (`tid` is short for \"test id\" and the authors of Moar use tids extensively for testing.)\n\n`Moar.File`\n\n- `new_tempfile_path` returns a new path for a tempfile, without creating it.\n- `write_tempfile` writes some data to a new tempfile.\n\n`Moar.Integer`\n\n- `compare` returning `:eq`, `:lt`, `:gt`.\n\n`Moar.List`\n\n- `to_keyword` converts a list into a keyword list, using a default value or function to generate the values\n- `to_sentence` converts a list into a string, with items separated by commas, and an \"and\" before the last item\n- `unwrap` returns the argument if it's not a list, or returns the only item in the list, or raises.\n\n`Moar.Map`\n\n- `atomize_key`, `atomize_keys`, and `deep_atomize_keys` convert keys in a map from strings to atoms, and `stringify_keys`\n  does the opposite.\n- `deep_take` takes a list of keys and `{key, nested_key}` tuples to take from nested maps.\n- `index_by` converts a list of maps into a map of maps indexed by the values of one of the keys.\n- `merge` and `deep_merge` merge maps, converting enumerables into maps before merging. `deep_merge` also accepts a\n  function to resolve value conflicts.\n- `merge_if_blank` merge maps, retaining existing non-blank values.\n- `put_if_blank` puts a key/value pair into a map if the key is missing or its value is blank (via `Moar.Term.blank?/1`)\n- `put_new!` is like `Map.put_new/3` but raises if the key already exists in the map.\n- `rename_key` and `rename_keys` rename keys in a map.\n- `transform` transforms a key or multiple keys with a transformer function.\n- `validate_keys!` validates that the keys in the map are equal to or a subset of a list of valid keys.\n\n`Moar.Opts`\n\n- is meant to be used with function options.\n- `get` extracts a value from opts, falling back to a default if the value is blank (via `Moar.Term.blank?/1`)\n- `take` extracts multiple values from opts, falling back to defaults if the value is blank (via `Moar.Term.blank?/1`)\n\n`Moar.Protocol`\n\n- `implements!` raises if a struct does not implement the given protocol.\n- `implements?` returns true if a struct implements the given protocol.\n\n`Moar.Random`\n\n- `float` returns a random float.\n- `integer` returns a random integer.\n- `string` returns random base64- or base32-encoded string.\n\n`Moar.Regex`\n\n- `named_capture` gets a single named capture.\n- `named_captures` is like `Regex.named_captures/3` but can take the first two args in any order.\n\n`Moar.Retry`\n\n- `rescue_until` and `rescue_for` run the given function repeatedly until it does not raise.\n- `retry_until` and `retry_for` run the given function repeatedly until it returns a truthy value.\n\n`Moar.String`\n\n- `append_unless_blank` appends a suffix to a string, unless the string is blank.\n- `compare` and `compare?` compare two strings, optionally transforming the strings before comparison.\n- `count_leading_spaces/1` returns the number of leading spaces in a string.\n- `inner_truncate` removes the middle of a string to make it the desired length.\n- `remove_marked_whitespace` removes whitespacing following a special `\\v` marker.\n- `secure_compare` compares two strings in constant time.\n- `slug` converts a string into a slug with a custom joiner character; `dasherize` and `underscore` are shortcuts for\n  common slug formats.\n- `squish` collapses consecutive whitespace characters.\n- `surround` wraps a string with the given characters.\n- `to_case` converts text to `:camel_case`, `:lower_camel_case`, or `:snake_case`.\n- `to_integer` converts a string to an integer with a few handy options.\n- `trim` is like `String.trim/1` but handles `nil` values.\n- `truncate_at` truncates a string at the last instance of a substring that results in the truncated string being shorter\n  than a given length.\n- `unindent/1` un-indents a multiline string by the smallest indentation size.\n- `unindent/2` un-indents a multiline string by the given amount.\n\n`Moar.Sugar`\n\n- `error`, `noreply`, and `ok` create tuples (`\"foo\" |> error()` -> `{:error, \"foo\"}`).\n- `error!` and `ok!` unwrap tuples (`{:error, \"foo\"} |> error!()` -> `\"foo\"`).\n- `returning` takes two arguments and returns the second one.\n\n`Moar.Term`\n\n- `blank?` returns true if a term is `nil`, an empty string, a whitespace-only string, an empty list, or an empty map.\n- `present?` is the opposite of `blank?`.\n- `presence` returns a default if the argument is blank.\n\n`Moar.Tuple`\n\n- `from_list!` converts a list of tuples like `[{:ok, 1}, {:ok, 2}, {:ok, 3}]` to a tuple like `{:ok, [1, 2, 3]}`\n- `reduce` converts a list of tuples like `[{:ok, 1}, {:error, 2}]` to a map like `%{ok: [1], error: [2]}`\n\n`Moar.URI`\n\n- `fix` applies some fixes to a URI string.\n- `format` formats a URI in various ways.\n- `valid?` returns true if the URI has a host and scheme, and if it has a path, the path does not contain spaces.\n- `web_url?` returns true if the scheme is `http` or `https`.\n\n`Moar.Version`\n\n- `compare` is like `Version.compare/2` but normalizes the versions first.\n- `normalize` appends as many \".0\" strings as necessary to create a string with major, minor, and patch numbers.","ref":"readme.html#a-quick-tour"},{"type":"extras","title":"LICENSE","doc":"Copyright 2022 synchronal.dev\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated\ndocumentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the\nrights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit\npersons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the\nSoftware.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE\nWARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\nCOPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR\nOTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}